/************************************************************************
 *
 *	VierVerliert
 *	(c) 2004, 2005 Thomas Unterthiner
 *
 * GameGrid.cpp - implementation of the game grid
 *
 ************************************************************************/

#include <cassert>
#include "GameGrid.h"


/*
 * Ctor...
 * initializes the game grid with a given number of rows and slots
 */
GameGrid::GameGrid(unsigned short rows, unsigned short slots)
	: rows_(rows),
	  slots_(slots)
{
	Slot tmp(rows, NO_PLAYER);
	data_.resize(slots, tmp);
}



/*
 * Dtor...
 */
GameGrid::~GameGrid()
{
}



/*
 * try to put a new coin into a slot
 *
 * search the "lowest" free place in the given slot and put
 * the coin there. If the slot is full, return false, otherwise
 * return true.
 */
bool GameGrid::putIntoSlot(unsigned short slot, PlayerCode player)
{
	assert(slot < slots_);
	int row = 0;
	while (data_[slot][row] != NO_PLAYER)
	{
		// slot is full
		if (row == rows_ - 1)
		{
			return false;
		}

		++row;
	}

	data_[slot][row] = player;
	return true;
}



/*
 * reset the grid to it's initial status
 */
void GameGrid::reset()
{
	for (int i = 0; i < slots_; ++i)
	{
		for (int j = 0; j < rows_; ++j)
		{
			data_[i][j] = NO_PLAYER;
		}
	}
}

