/************************************************************************
 *
 *	VierVerliert
 *	(c) 2004, 2005 Thomas Unterthiner
 *
 * ScoreController.cpp - controls if and who has won the game
 *
 ************************************************************************/


#include "ScoreController.h"
#include "GameGrid.h"



/*
 * Ctor...
 */
ScoreController::ScoreController(const GameGrid& grid, int loosingPoints)
	: loosingPoints_(loosingPoints),
	  loosingScorePlayer1(loosingPoints * PLAYER_1),
	  loosingScorePlayer2(loosingPoints * PLAYER_2),
	  grid_(grid)
{
}



/*
 * search for a winner. If there isn't one, return
 * NO_PLAYER, otherwhise return the code of the winner
 */
PlayerCode ScoreController::checkForWinner()
{
	PlayerCode status = NO_PLAYER;

	if (endedWithDraw())
	{
		return NO_WINNER;
	}

	status = checkVertically();
	if (status != NO_PLAYER) return status;

	status = checkHorizontally();
	if (status != NO_PLAYER) return status;

	status = checkDiagonally1();
	if (status != NO_PLAYER) return status;

	return checkDiagonally2();
}



/*
 * check for vertical wins, returns the PlayerCode of the winner
 */
PlayerCode ScoreController::checkVertically() const
{
	int points1 = 0;
	int points2 = 0;
	for (int i = 0; i <  grid_.getSlots(); ++i)
	{
		for (int j = 0; j < grid_.getRows(); ++j)
		{
			switch(grid_.get(i, j))
			{
				case PLAYER_1:
					points1 += PLAYER_1;
					points2 = 0;
					break;
				case PLAYER_2:
					points2 += PLAYER_2;
					points1 = 0;
					break;
				default:
					points1 = 0;
					points2 = 0;
			}

			if (points1 == loosingScorePlayer1)
				return PLAYER_2;
			else if (points2 == loosingScorePlayer2)
				return PLAYER_1;
		}
		points1 = 0;
		points2 = 0;
	}

	return NO_PLAYER;
}



/*
 * check for horizontal wins
 */
PlayerCode ScoreController::checkHorizontally () const
{
	int points1 = 0;
	int points2 = 0;
	for (int i = 0; i < grid_.getRows(); ++i)
	{
		for (int j = 0; j < grid_.getSlots(); ++j)
		{
			switch(grid_.get(j, i))
			{
				case PLAYER_1:
					points1 += PLAYER_1;
					points2 = 0;
					break;
				case PLAYER_2:
					points2 += PLAYER_2;
					points1 = 0;
					break;
				default:
					points1 = 0;
					points2 = 0;
			}

			if (points1 == loosingScorePlayer1)
				return PLAYER_2;
			else if (points2 == loosingScorePlayer2)
				return PLAYER_1;
		}
		points1 = 0;
		points2 = 0;
	}

	return NO_PLAYER;
}



/*
 * check for diagonal wins from the left lower to the right upper
 */
PlayerCode ScoreController::checkDiagonally1() const
{
	int sum = 0;

	for (int i = 0; i <= grid_.getSlots() - loosingPoints_; ++i)
	{
		for (int j = 0; j <= grid_.getRows() - loosingPoints_; ++j)
		{
			sum = grid_.get(i, j);
			sum += grid_.get(i+1, j+1);
			sum += grid_.get(i+2, j+2);
			sum += grid_.get(i+3, j+3);

			if (sum == loosingScorePlayer1)
			{
				return PLAYER_2;
			}
			else if (sum == loosingScorePlayer2)
			{
				return PLAYER_1;
			}
		}
	}
	return NO_PLAYER;
}



/*
 * check for diagonal wins from the lower right to the upper left
 */
PlayerCode ScoreController::checkDiagonally2() const
{
	int sum = 0;
	for (int i = grid_.getSlots() - 1; i >= loosingPoints_ - 1; --i)
	{
		for (int j = 0; j <= grid_.getRows() - loosingPoints_; ++j)
		{
			sum = grid_.get(i, j);
			sum +=  grid_.get(i-1, j+1);
			sum +=  grid_.get(i-2, j+2);
			sum +=  grid_.get(i-3, j+3);

			if (sum == loosingScorePlayer1)
			{
				return PLAYER_2;
			}
			else if (sum == loosingScorePlayer2)
			{
				return PLAYER_1;
			}
		}
	}
	return NO_PLAYER;
}



/*
 * control if the game ended in a draw
 * if there aren't any more free places on the grid and we haven't a winner
 * yet, than the game has finished without a winner.
 *
 * Note that this check could also have been implemented within the other
 * checkXXX() methods, but I think it's semantically clearer to implement
 * it on its own...
 */
bool ScoreController::endedWithDraw() const
{
	for (int i = 0; i < grid_.getSlots(); ++i)
	{
		for (int j = 0; j < grid_.getRows(); ++j)
		{
			if (grid_.get(i, j) == NO_PLAYER)
			{
				return false;
			}
		}
	}

	return true;
}
