#ifndef SCORECONTROLLER_H
#define SCORECONTROLLER_H
/************************************************************************
 *
 *	VierVerliert
 *	(c) 2004, 2005 Thomas Unterthiner
 *
 * ScoreController.h - controls if and who has won the game
 *
 ************************************************************************/


#include "PlayerCode.h"

class GameGrid;



/*
 * The ScoreController has to check if there's a winner in the game.
 * As this is VierVerliert, the winner is the one who would
 * actually loose in a game of the well-known Vier Gewinnt game.
 *
 * The Controller will look for 4 coins of the same player
 * in a horizontal, vertical and diagonal line on the grid.
 * If it finds one such line, then that player has lost!
 *
 * Also, if no free slot could be found (and the game therefore
 * ended in a draw), NO_WINNER will be returned.
 *
 * Note that the algorithms used here depend on the definition of
 * PlayerCode: PLAYER_1 * loosingPoints HAS TO BE lower than
 * PLAYER_2 * 1
 */
class ScoreController
{
	public:
		ScoreController(const GameGrid& grid, int loosingPoints = 4);

		PlayerCode checkForWinner();


	private:
		PlayerCode checkVertically() const;
		PlayerCode checkHorizontally() const;
		PlayerCode checkDiagonally1() const;
		PlayerCode checkDiagonally2() const;
		bool endedWithDraw() const;

		const int loosingPoints_;
		const int loosingScorePlayer1;
		const int loosingScorePlayer2;
		const GameGrid& grid_;
};

#endif

