/************************************************************************
 *
 *	VierVerliert
 *	(c) 2004, 2005 Thomas Unterthiner
 *
 * SdlFont.cpp - a simple wrapper over SDL_ttf
 *
 ************************************************************************/

#include "SdlFont.h"

#include <string>
#include <vector>
#include <cassert>
using namespace std;

#include <SDL/SDL_ttf.h>
#include <SDL.h>


/*
 * Ctor
 * note that this Ctor is private, and is intended to be
 * called only from SdlFontManger, which deals with
 * loding the TTF_Fonts.
 */
SdlFont::SdlFont(TTF_Font* font)
	: font_(font)
{
	assert(font_);
}


/*
 * renders a given text on a Surface with alpha.
 */
SDL_Surface* SdlFont::renderBlended(const string& text, Uint8 r, Uint8 g, Uint8 b) const
{
	SDL_Color color = {r, g, b};
	SDL_Surface* retval = TTF_RenderText_Blended(font_, text.c_str(), color);
	assert(retval);

	return retval;
}



/*
 * renders text that is more than 1 line long.
 * The text will be splitted on every newline, and every line will be
 * rendered on its own, than those parts will all be blitted on the
 * final surface.
 */
SDL_Surface* SdlFont::renderMultilineBlended(const string& text,
                                             Uint8 r, Uint8 g, Uint8 b,
                                             Alignment align) const
{
	vector<string> lines = splitMultiline(text);
	vector<SDL_Surface*> surfaces;
	int finalHeight = 0;
	int finalWidth = 0;

	// create the single surfaces
	for (int i = 0; i < lines.size(); ++i)
		surfaces.push_back(renderBlended(lines[i], r, g, b));

	SDL_Surface* retval = blitOnOneSurface(surfaces, align);

	for (int i = 0; i < lines.size(); ++i)
		SDL_FreeSurface(surfaces[i]);

	return retval;
}



/*
 * splits a line on newline
 */
vector<string> SdlFont::splitMultiline(const string& text) const
{
	string::size_type startPos = 0;
	string::size_type endPos = 0;
	vector<string> lines;

	while (endPos != string::npos)
	{
		endPos = text.find_first_of('\n', startPos);
		lines.push_back(text.substr(startPos, endPos - startPos));
		startPos = endPos + 1;
	}

	return lines;
}

