#ifndef SDLFONTMANAGER_H
#define SDLFONTMANAGER_H
/************************************************************************
 *
 *	VierVerliert
 *	(c) 2004, 2005 Thomas Unterthiner
 *
 * SdlFontManager.h - a simple wrapper over SDL_ttf
 *
 ************************************************************************/
#include <map>
#include <string>
#include <stdexcept>
using std::string;
using std::map;

#include "SdlFont.h"


/*
 * implements a very simple FontManager as a Singleton
 * internally all fonts ever loaded will be stored in a lookuptable.
 * As the key for the LUT has to be constructed from the Name of the Font.
 * 
 * Note that loading fonts and rendering multiline texts should not be
 * called within performance-critical code!
 */
class SdlFontManager
{
	public:

		static SdlFontManager& get();
		SdlFont& getFont(const string name, int size);

		~SdlFontManager();
	private:
		SdlFontManager();
		string makeKey(const string& name, int size);
		
		typedef map<string, SdlFont*> FontMap;

		bool quitFontSubsystemOnExit_;
		FontMap fonts_;
};


#endif

