/************************************************************************
 *
 *	VierVerliert
 *	(c) 2004, 2005 Thomas Unterthiner
 *
 * SdlGridDrawer.cpp - implements a drawer using libSDL
 *
 ************************************************************************/

#include "SdlGridDrawer.h"

#include <SDL.h>
#include <cassert>
#include "PlayerCode.h"
#include "GameGrid.h"
#include "SdlHelper.h"

/*
 * Background fill color
 */
const Uint8 BACKGROUND_R = 0;
const Uint8 BACKGROUND_G = 0;
const Uint8 BACKGROUND_B = 255;

const int CURSOR_SPACE = 60; // space to be left free on the upper side for the cursor in px



/*
 * Ctor
 */
SdlGridDrawer::SdlGridDrawer(const GameGrid& grid, const Player& p1,
                             const Player& p2, SDL_Surface* display)
	: grid_(grid),
	  display_(display),
	  overlay_(loadImage("overlay.bmp", true)),
	  cursor_(loadImage("cursor.bmp", true)),
	  startDrawingOffsetX((display_->w - (overlay_->w * grid_.getSlots())) / 2),
	  startDrawingOffsetY((display_->h - CURSOR_SPACE - overlay_->h * grid_.getRows()) / 2)
{
	assert(display->h >= CURSOR_SPACE + overlay_->h * grid_.getRows());
	assert(display->w >= overlay_->w * grid_.getSlots());
	colorPlayer1_ = SDL_MapRGB(display_->format, p1.r, p1.g, p1.b);
	colorPlayer2_ = SDL_MapRGB(display_->format, p2.r, p2.g, p2.b);
	colorNoPlayer_ = SDL_MapRGB(display_->format, 255, 255, 255);
}


/*
 * Dtor
 */
SdlGridDrawer::~SdlGridDrawer()
{
	SDL_FreeSurface(cursor_);
	SDL_FreeSurface(overlay_);
}



/*
 * draw the current status of the grid onto display_
 */
void SdlGridDrawer::draw()
{
	SDL_FillRect(display_, 0,
		SDL_MapRGB(display_->format, BACKGROUND_R, BACKGROUND_G, BACKGROUND_B));
	SDL_Rect drawPosition;
	drawPosition.w = overlay_->w;
	drawPosition.h = overlay_->h;
	
	drawCursor();

	for (int i = 0; i < grid_.getSlots(); ++i)
	{
		drawPosition.x = startDrawingOffsetX + overlay_->w * i;
		for (int j = 0; j < grid_.getRows(); ++j)
		{
			drawPosition.y = display_->h - startDrawingOffsetY -  (j+1) * overlay_->h;
			switch (grid_.get(i, j))
			{
				case PLAYER_1:
					SDL_FillRect(display_, &drawPosition, colorPlayer1_);
					break;
				case PLAYER_2:
					SDL_FillRect(display_, &drawPosition, colorPlayer2_);
					break;
				case NO_PLAYER:
					SDL_FillRect(display_, &drawPosition, colorNoPlayer_);
					break;
			}
			SDL_BlitSurface(overlay_, 0, display_, &drawPosition);
		}
	}

	SDL_Flip(display_);
}



/*
 * draws a cursor over the selected slot
 */
void SdlGridDrawer::drawCursor()
{
	// clear old cursor
	SDL_Rect rect;
	rect.x = rect.y = 0;
	rect.w = display_->w;
	rect.h = CURSOR_SPACE;
	SDL_FillRect(display_, &rect, SDL_MapRGB(display_->format, 255, 255, 255));

	rect.y = (CURSOR_SPACE - cursor_->h) / 2;
	rect.x = startDrawingOffsetX + cursorSlot_ * overlay_->w;
	SDL_BlitSurface(cursor_, 0, display_, &rect);
}



/*
 * simple setter
 */
void SdlGridDrawer::setCursorToSlot(unsigned short slot)
{
	assert(slot < grid_.getSlots());
	cursorSlot_ = slot;
}
