/***********************************************************************
 *
 *	VierVerliert
 *	(c) 2004, 2005 Thomas Unterthiner
 *
 * TextUserInterface.cpp - interacts with the user through the console
 *
 ************************************************************************/


#include <iostream>
using std::cin;
using std::cout;

#include "TextUserInterface.h"
#include "GameManager.h"
#include "ConsoleGridDrawer.h"


/*
 * Ctor...
 */
TextUserInterface::TextUserInterface(GameManager& game)
	: UserInterface(game)
{
	drawer_ = new ConsoleGridDrawer(game.getGrid(), game.getPlayer1(), game.getPlayer2());
}


/*
 * Dtor
 */
TextUserInterface::~TextUserInterface()
{
	delete drawer_;
}

/*
 * ask the player where he wants to insert the  
 * coin and then  tell the game about it
 *
 * the user can input some value between
 * [1, NrOfSlots], but the input will be 
 * edited to fit into [0, NrOfSlots-1]
 *
 */
void TextUserInterface::handleInput()
{
	drawer_->draw();
	int chosenSlot = 0;
	cout << getGame().getActualPlayer().name << ", it's your turn!\n";

	while ((cout << "where would you like to place the next coin: ")  && !(cin >> chosenSlot)
		|| chosenSlot <= 0 || chosenSlot > getGame().getNrOfSlots())
	{
		cout << "\n!!! sorry, but that's not a valid slot !!!\n\n\n";
		resetStdin();
	}

	--chosenSlot;

	try
	{
		getGame().putIntoSlot(chosenSlot);
	}
	catch(GameManager::IllegalSlotError)
	{
		cout << "\n!!! sorry, but that slot's already full !!!\n\n\n";
		handleInput();
	}
}



/*
 * notify the user about who has won the game
 */
void TextUserInterface::showWinner(const Player& winner)
{
	if (winner.code != NO_WINNER)
	{
		cout << "Congratulations ";
		cout << winner.name << ", you have won!\n\n";
	}
	else
	{
		cout << "the game ended in a draw!\n\n";
	}
	
	resetStdin();
	cin.get();
}


/*
 * shows the main menu... basically just a simple startup screen here
 *
 * note that if this function returns without calling game_->run() the
 * actual game won't start, but the program will simply stop executing
 */
void TextUserInterface::showMainMenu()
{
	cout << "---------------------------------------------\n\n";
	cout << "              VierVerliert                   \n\n\n";
	cout << "    (c) 2004, 2005 Thomas Unterthiner        \n\n";
	cout << "---------------------------------------------\n\n\n";
	cout << "press\n";
	cout << "	[0] to exit\n";
	cout << "	[1] to play\n\n";
	cout << "also note that you can quit the game at any time\n";
	cout << "by pressing CTRL+C (Windows) / CTRL+Z (Linux)\n\n";

	int choice = -1;
	while(cout << "please make your selection: " && !(cin >> choice)
		&& (choice != 0 || choice != 1))
	{
		cout << "sorry, but this is not allowed! ;-)\n\n";
		resetStdin();
	}
	
	if (choice == 0)
		getGame().exitProgram();
	else
		getGame().run();
}




/*
 * if for any reason cin's status is bad, reset cin here
 * A reason for cin to fail could be the user entering a string
 * where an integer was expected
 */
void TextUserInterface::resetStdin()
{
	cin.clear();
	cin.sync();
	cin.ignore(cin.rdbuf()->in_avail());
	// just to make sure cin is completely EMTPY
	while (cin.get() != '\n')
		;
}
