#include "../ScoreController.cpp"
#include "../GameGrid.cpp"

#include <iostream>
#include <vector>
using namespace std;



/*
 * should simply check that ctor and dtor don't crash
 */
bool test0()
{
	GameGrid grid(2, 2);
	ScoreController* sc = new ScoreController(grid);
	delete sc;
	return true;
}


/*
 * should trigger ScoreController::checkDiagonally1() to return PLAYER_1
 */
bool test1()
{
	GameGrid grid(8, 8);
	ScoreController sc(grid);

	grid.putIntoSlot(4, PLAYER_2);

	grid.putIntoSlot(5, PLAYER_1);
	grid.putIntoSlot(5, PLAYER_2);

	grid.putIntoSlot(6, PLAYER_1);
	grid.putIntoSlot(6, PLAYER_1);
	grid.putIntoSlot(6, PLAYER_2);

	grid.putIntoSlot(7, PLAYER_1);
	grid.putIntoSlot(7, PLAYER_1);
	grid.putIntoSlot(7, PLAYER_1);
	grid.putIntoSlot(7, PLAYER_2);

	return sc.checkForWinner() == PLAYER_1;
}


/*
 * should trigger ScoreController::checkDiagonally1() to return PLAYER_1
 * but on a field that is only as small as possible
 */
bool test2()
{
	GameGrid grid(4, 4);
	ScoreController sc(grid);

	grid.putIntoSlot(0, PLAYER_2);

	grid.putIntoSlot(1, PLAYER_1);
	grid.putIntoSlot(1, PLAYER_2);

	grid.putIntoSlot(2, PLAYER_1);
	grid.putIntoSlot(2, PLAYER_1);
	grid.putIntoSlot(2, PLAYER_2);

	grid.putIntoSlot(3, PLAYER_1);
	grid.putIntoSlot(3, PLAYER_1);
	grid.putIntoSlot(3, PLAYER_1);
	grid.putIntoSlot(3, PLAYER_2);
	
	return sc.checkForWinner() == PLAYER_1;
}



/*
 * should trigger ScoreController::checkDiagonally2() to return PLAYER_2
 */
bool test3()
{
	GameGrid grid(5, 5);
	ScoreController sc(grid);

	grid.putIntoSlot(0, PLAYER_1);
	grid.putIntoSlot(0, PLAYER_1);
	grid.putIntoSlot(0, PLAYER_1);
	grid.putIntoSlot(0, PLAYER_2);

	grid.putIntoSlot(1, PLAYER_1);
	grid.putIntoSlot(1, PLAYER_1);
	grid.putIntoSlot(1, PLAYER_2);

	grid.putIntoSlot(2, PLAYER_1);
	grid.putIntoSlot(2, PLAYER_2);

	grid.putIntoSlot(3, PLAYER_2);
	
	return sc.checkForWinner() == PLAYER_1;
}



/*
 * should trigger ScoreController::checkDiagonally2() to return PLAYER_2
 * but on a field that is as small as possible
 */
bool test4()
{
	GameGrid grid(4, 4);
	ScoreController sc(grid);

	grid.putIntoSlot(0, PLAYER_1);
	grid.putIntoSlot(0, PLAYER_1);
	grid.putIntoSlot(0, PLAYER_1);
	grid.putIntoSlot(0, PLAYER_2);

	grid.putIntoSlot(1, PLAYER_1);
	grid.putIntoSlot(1, PLAYER_1);
	grid.putIntoSlot(1, PLAYER_2);

	grid.putIntoSlot(2, PLAYER_1);
	grid.putIntoSlot(2, PLAYER_2);

	grid.putIntoSlot(3, PLAYER_2);

	return sc.checkForWinner() == PLAYER_1;
}

/*
 * should trigger ScoreController::checkDiagonally2() to return PLAYER_2
 * but with the triggering coins placed as right as possible
 */
bool test5()
{
	GameGrid grid(8, 8);
	ScoreController sc(grid);

	grid.putIntoSlot(4, PLAYER_1);
	grid.putIntoSlot(4, PLAYER_1);
	grid.putIntoSlot(4, PLAYER_1);
	grid.putIntoSlot(4, PLAYER_2);

	grid.putIntoSlot(5, PLAYER_1);
	grid.putIntoSlot(5, PLAYER_1);
	grid.putIntoSlot(5, PLAYER_2);

	grid.putIntoSlot(6, PLAYER_1);
	grid.putIntoSlot(6, PLAYER_2);

	grid.putIntoSlot(7, PLAYER_2);
	
	return sc.checkForWinner() == PLAYER_1;
}


bool testHorizontal()
{
	GameGrid grid(4, 4);
	ScoreController sc(grid);
	grid.putIntoSlot(0, PLAYER_1);
	grid.putIntoSlot(1, PLAYER_1);
	grid.putIntoSlot(2, PLAYER_1);
	grid.putIntoSlot(3, PLAYER_1);

	return sc.checkForWinner() == PLAYER_2;
}



bool testVertical()
{
	GameGrid grid(4, 4);
	ScoreController sc(grid);
	grid.putIntoSlot(0, PLAYER_1);
	grid.putIntoSlot(1, PLAYER_1);
	grid.putIntoSlot(2, PLAYER_1);
	grid.putIntoSlot(3, PLAYER_1);

	return sc.checkForWinner() == PLAYER_2;
}




int main()
{
	bool (*tests[])() = {test0, test1, test2, test3, test4, test5, testVertical, testHorizontal};

	int nErrors = 0;
	for (int i = 0; i < sizeof(tests)/sizeof(tests[0]); ++i)
	{
		if ((*tests[i])() == false)
		{
			++nErrors;
			cout << "ERROR " << nErrors << ": test " << i << " failed\n";
		}
		else
			cout << "test " << i << " passed\n";
	}
	
	cout << "\n\nthere were " << nErrors << " errors\n";

}
